from __future__ import print_function
try:
	from jira import JIRA
except ImportError:
	print('Please install jira with pip install jira')
try:
	from git import Repo
except ImportError:
	print('Please install gitpython with pip install gitpython')
try:
	import requests
except ImportError:
	print('Please install requests with pip install requests')

import re
import os
from datetime import datetime
import random

class JiraReleaser(object):
	JIRA_ISSUE_REGEX = '[A-Z]+(?!-?[a-zA-Z]{1,10})-\d+'

	def create_jira(self, jira_base_url, jira_username, jira_password):
		"""
		"""
		self.jira = JIRA(jira_base_url, basic_auth=(jira_username, jira_password))

	def release_version(self):
		"""
		"""
		version = None
		for project_version in self.jira.project_versions(self.project_key):
			if project_version.name == self.version_name:
				version = project_version

	def list_commits(self):
		"""
		"""
		return list(self.repo.iter_commits('{}..HEAD'.format(self.from_commit)))

	def get_issues(self, commits):
		"""
		"""
		issues = []
		for commit in commits:
			issues.extend(re.findall(self.JIRA_ISSUE_REGEX, commit.message))
		return set(issues)

	def update_issues_version(self, issues):
		"""
		"""
		for issue in issues:
			item = self.jira.issue(issue)
			fixVersions = [{'name': self.version_name}]
			for fix in item.fields.fixVersions:
				fixVersions.append({'name': fix.name})
			item.update(fields={"fixVersions": fixVersions})
		return '{} updated'.format(issues)

	def update_issues_labels(self, issues):
		"""
		"""
		for issue in issues:
			item = self.jira.issue(issue)
			label = self.create_jira_label().encode('utf-8')
			item.fields.labels.append(label)
			item.update(fields={"labels": item.fields.labels})
		return '{} updated'.format(issues)

	def create_jira_version(self):
		"""
		"""
		return self.jira.create_version(name=self.create_jira_version_name(), project=self.project_key,releaseDate=str(datetime.now()))

	def create_jira_version_name(self):
		"""
		"""
		if self.is_development:
			return self.project_name + '-dev-' + str(self.build_number)
		elif self.is_staging:
			return self.project_name + '-staging-' + str(self.build_number)
		elif self.is_production:
			return self.project_name + '-production-' + str(self.build_number)

	def release_version(self, version):
		version.update(fields={'released': True})

	def __str__(self):
		return 'Project .git: {}\nProject key: {}'.format(self.repo.working_tree_dir, self.project_key)
	def __unicode__(self):
		return u'Project .git: {}\nProject key: {}'.format(self.repo.working_tree_dir, self.project_key)

	def __init__(self, project_key, jira_url, jira_username=None, jira_password=None, project_name=None, from_commit=None, to_commit=None, build_number=None, is_development=False, is_staging=False, is_production=False, repo_root=None):
		self.project_key = project_key
		self.from_commit = from_commit
		self.to_commit = to_commit
		self.project_name = project_name
		self.build_number = build_number
		self.is_development = is_development
		self.is_staging = is_staging
		self.is_production = is_production
		if not repo_root:
			self.repo = Repo(os.path.dirname(os.path.realpath(__file__)))
		else:
			self.repo = Repo(repo_root)
		self.create_jira(jira_url, jira_username, jira_password)